<?php

/*
This PHP cli helper script will upload existing media files
to OpenStack object store and update links witin posts

Berore running it make sure that:
1. You have database and media files backups
2. swift-storage plugin is configured and can reach swift.
   Good test is to check if swift buckets are showing on 
   the plugin configuration page
3. Swift bucket is publically readable ie.
   swift post -r '.r:*' WordPress
4. Swift URLs are reachable from outside world 

The script must be run from the top level directory holding
wordpress code for example:

cd /srv/wordpress
php wp-content/plugins/swift-storage/migrate.php

*/

@include "wp-config.php";
@include_once "wp-includes/option.php";
@include_once "wp-includes/functions.php";
@include_once "wp-includes/post.php";

$plugin_path = dirname($argv[0]);
require_once $plugin_path . "/classes/swift.php";

if($argv[1] != "--break-wordpress") {
    print "Make sure you have backups and you know what you are doing.\n";
    print "If you really want to migrate data rerun the script with --break-wordpress switch\n";
    exit;
}

$uploaded_files = 0;
global $wpdb;
$siteurl = $wpdb->get_var("select option_value from wp_options where option_name = 'siteurl'");
$last_post = $wpdb->get_var("SELECT MAX(ID) from wp_posts");
$swift = new Swift( __FILE__ );

function fix_space($post_id) {
    $filename = get_post_meta( $post_id, $key = '_wp_attached_file', true);
    $filename = str_replace( ' ', '+', $filename);
    delete_post_meta($post_id, '_wp_attached_file');
    add_post_meta($post_id, '_wp_attached_file', $filename);
    $meta = get_post_meta( $post_id, $key = '_wp_attachment_metadata', true);
    $meta['file'] = str_replace( ' ', '+', $meta['file']);
    delete_post_meta($post_id, '_wp_attachment_metadata');
    add_post_meta($post_id, '_wp_attachment_metadata', $meta);
    $swift_info = get_post_meta( $post_id, $key = 'swift_info', true);
    $swift_info['key'] = str_replace( ' ', '+', $swift_info['key']);
    delete_post_meta($post_id, 'swift_info');
    add_post_meta($post_id, 'swift_info', $swift_info);
}

for($i=0; $i<=$last_post; $i++) {
    $filename = get_post_meta( $i, $key = '_wp_attached_file', true);
    $meta = get_post_meta( $i, $key = '_wp_attachment_metadata', true);

    // Some files can have full paths, we have to normalize it
    if(preg_match('/\/srv\//', $filename)) {
        print "\tUpdating path for file: " . $filename . "\n";
        $filename = preg_replace( '/.*\/wp-content\/uploads\//', '', $filename);
        delete_post_meta($i, '_wp_attached_file');
        add_post_meta($i, '_wp_attached_file', $filename);
    }
    if(!empty($meta)) {
        if(preg_match('/\/srv\//', $meta['file'])) {
            $meta['file'] = preg_replace( '/.*\/wp-content\/uploads\//', '', $meta['file']);
            delete_post_meta($i, '_wp_attachment_metadata');
            add_post_meta($i, '_wp_attachment_metadata', $meta);
        }
    }
    if(!empty($filename) && !empty($meta)){
        $old_url = $siteurl . "/wp-content/uploads/" . $filename;
        $swift->wp_update_attachment_metadata($meta, $i);
        if (strpos($filename, ' ') !== FALSE) {
            print "\tDetected space in filename: " . $filename . " replacing with \"+\"\n";
            fix_space($i);
        }
        $new_url = $swift->swift_get_attachment_url($i);
        $_tmp = @file_get_contents($new_url);
        if(empty($_tmp)){
            die("\nERROR: Can't verify upload for " . $filename . " (url:  " . $new_url . "). Giving up!\n");
        }
        print $old_url . " => " . $new_url . "\n";
        $uploaded_files++;
        $sql = "UPDATE wp_posts SET post_content = REPLACE(post_content, '" . $old_url . "', '" . $new_url . "')";
        //print "SQL query: " . $sql . "\n";
        $wpdb->get_var($sql);
    }
}

print "\n\n#################################\n# Number of files uploaded: " . $uploaded_files . "\n\n";

